package net.ibizsys.central.plugin.boot.core.dict;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.annotation.JSONField;
import com.alibaba.fastjson.util.TypeUtils;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;
import lombok.experimental.Accessors;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;

import java.util.*;

@Getter
@Setter
@NoArgsConstructor
@AllArgsConstructor
@Accessors(chain = true)
public class Catalog
{
	private String code;
	private String name;

	@JSONField(name = "items")
	@JsonProperty("items")
	private List<Option> options = new ArrayList<>();

	public Option findCodeItem(Object value)
	{
		return findCodeItem(value,this.options);
	}

	public Option findCodeItem(Object value, List<Option> options){
		for(Option codeItem : options){
			if(value != null && codeItem.getValue() != null && value.equals(codeItem.getValue()))
				return codeItem;
			else if (!ObjectUtils.isEmpty(codeItem.getChildren()))
			{
				Option rt=findCodeItem(value,codeItem.getChildren());
				if(rt!=null)
					return rt;
			}
		}
		return null;
	}

	private Map<String, Object> advancedSettings;

	public Catalog putAdvancedSettings(String settings)
	{
		try
		{
			if(!StringUtils.isEmpty(settings))
				advancedSettings= JSON.parseObject(settings, Map.class);
		}catch (Exception ex){}
		return this;
	}

	@JsonIgnore
	@JSONField(serialize = false)
	public String getUrl() {
		return getSetting("url",null);
	}


	@JsonIgnore
	@JSONField(serialize = false)
	public String getRequestMethod() {
		return getSetting("requestMethod","GET");
	}

	@JsonIgnore
	@JSONField(serialize = false)
	public String getRequestBody() {
		return getSetting("requestBody","{}");
	}

	@JsonIgnore
	@JSONField(serialize = false)
	public String getNodeProp() {
		return getSetting("nodeProp","");
	}

	@JsonIgnore
	@JSONField(serialize = false)
	public String getValueProp() {
		return getSetting("valueProp","value");
	}

	@JsonIgnore
	@JSONField(serialize = false)
	public String getLabelProp() {
		return getSetting("labelProp","label");
	}

	@JsonIgnore
	@JSONField(serialize = false)
	public String getParentProp() {
		return getSetting("parentProp","parent");
	}

	@JsonIgnore
	@JSONField(serialize = false)
	public String getChildrenProp() {
		return getSetting("childrenProp","children");
	}


	@JsonIgnore
	@JSONField(serialize = false)
	public String getDisabledProp() {
		return getSetting("disabledProp","disabled");
	}

	@JsonIgnore
	@JSONField(serialize = false)
	public String getFilterProp() {
		return getSetting("filterProp","filter");
	}

	@JsonIgnore
	@JSONField(serialize = false)
	public String getExpiredProp() {
		return getSetting("expiredProp","expired");
	}

	@JsonIgnore
	@JSONField(serialize = false)
	public String getIconClassProp() {
		return getSetting("iconClassProp","iconClass");
	}




	public Catalog setNodes(List<Map> nodes)
	{
		return setNodes(nodes,null,null);
	}

	public Catalog setNodes(List<Map> nodes, String filter, String subCode)
	{
		Map<String, List<Option>> map = new LinkedHashMap<>();
		Option subOption=null;
		for(Map item:unfoldList(nodes,null))
		{
			Map<String, Object> extension = new HashMap<>();
			if(!StringUtils.isEmpty(item.get("extension")))
				extension = JSONObject.parseObject(item.get("extension").toString(), Map.class);
			String pid= getStringValue(item.get(this.getParentProp()), getStringValue(item.get("pvalue"),"_root"));
			if(StringUtils.isEmpty(pid))pid="_root";
			List<Option> list=null;
			if(!map.containsKey(pid))
			{
				list=new ArrayList<>();
				map.put(pid,list);
			}
			else
				list=map.get(pid);
			Option option=new Option().setValue(getStringValue(item.get(this.getValueProp()), getStringValue(item.get("id"),null)))
					.setId(getStringValue(item.get(this.getValueProp()), getStringValue(item.get("id"),null)))
					.setDisabled(getBooleanValue(item.get(this.getDisabledProp()),false)|| getBooleanValue(item.get(this.getExpiredProp()),false))
					.setFilter(getStringValue(item.get(this.getFilterProp()),null)).setIconClass(getStringValue(item.get(this.getIconClassProp()),null))
					.setLabel(getStringValue(item.get(this.getLabelProp()), getStringValue(item.get("text"),"")))
					.setParent(getStringValue(item.get(this.getParentProp()), getStringValue(item.get("pvalue"),""))).setExtension(extension);
			if(option.getValue().equals(subCode))
				subOption=option;
			if(StringUtils.isEmpty(filter)||option.getFilterSet().contains(filter))
				list.add(option);
		}
		List<Option> codeItemTreeList = loop(map, (StringUtils.isEmpty(subCode))?"_root":subCode);
		if(subOption!=null)
		{
			subOption.setChildren(codeItemTreeList).setParent("");
			this.getOptions().add(subOption.setParent(null));
		}
		else
			this.setOptions(codeItemTreeList);
		return this;
	}

	private List<Map> unfoldList(List<Map> nodes, String parent)
	{
		List<Map> unfoldList = new ArrayList<>();
		for(Map item:nodes)
		{
			if((!StringUtils.isEmpty(parent))&& StringUtils.isEmpty(getStringValue(item.get(this.getParentProp()), getStringValue(item.get("pvalue"),null))))
				item.put(this.getParentProp(),parent);
			String value = getStringValue(item.get(this.getValueProp()), getStringValue(item.get("id"),null));
			unfoldList.add(item);
			Object child=item.get(this.getChildrenProp());
			if(!ObjectUtils.isEmpty(child))
			{
				if(child instanceof List && ((List) child).get(0) instanceof Map)
					unfoldList.addAll(unfoldList((List)child,value));
				item.remove(this.getChildrenProp());
			}
		}
		return unfoldList;
	}

	private List<Option> loop(Map<String, List<Option>> listCodeItem, Object parentValue) {
		List<Option> trees = new ArrayList<Option>();
		if(listCodeItem.containsKey(parentValue)) {
			for (Option codeItem : listCodeItem.get(parentValue)) {
				List<Option> childCodeItem = loop(listCodeItem, codeItem.getValue());
				if (childCodeItem.size() > 0)
					codeItem.setChildren(childCodeItem);
				trees.add(codeItem);
			}
		}
		return trees;
	}


	private String getSetting(String key,String defaultValue)
	{
		return advancedSettings!=null&&advancedSettings.get(key)!=null?advancedSettings.get(key).toString():defaultValue;
	}

	public static <V> V getValue(Object objValue,Class<V> clazz,Object defaultValue)
	{
		if(objValue==null) return defaultValue==null?null:(V)defaultValue;
		return TypeUtils.castToJavaBean(objValue,clazz);
	}

	public static String getStringValue(Object objValue,String defaultValue)
	{
		return getValue(objValue,String.class,defaultValue);
	}

	public static Boolean getBooleanValue(Object objValue,Boolean defaultValue)
	{
		return getValue(objValue,Boolean.class,defaultValue);
	}
}
