package net.ibizsys.central.plugin.boot.core.runtime;

import com.alibaba.fastjson.annotation.JSONField;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.boot.context.properties.NestedConfigurationProperty;
import org.springframework.stereotype.Component;

import java.util.LinkedHashMap;
import java.util.Map;

@Data
@NoArgsConstructor
@AllArgsConstructor
@Component
@ConfigurationProperties(prefix="ibiz")
public class BootDeploySystems{

    private Map<String,BootDeploySystem> deploySystems=new LinkedHashMap<>();

    public void setDeploySystems(Map<String, BootDeploySystem> deploySystems) {
        if(deploySystems!=null)
            deploySystems.entrySet().forEach(entry->{
                entry.getValue().setDeploySystemId(entry.getKey());
            });
        this.deploySystems = deploySystems;
    }

    public BootDeploySystem get(String systemId) {
        return getDeploySystem(systemId);
    }

    public BootDeploySystem getDeploySystem(String systemId) {
        return deploySystems.get(systemId);
    }

    @Data
    @NoArgsConstructor
    @AllArgsConstructor
    public static class BootDeploySystem {
        @JsonProperty("deploysystemid")
        @JSONField(name = "deploysystemid")
        private String deploySystemId;

        @JsonProperty("runtimeclassname")
        @JSONField(name = "runtimeclassname")
        private String runtimeClassName;

        @JsonProperty("modelpath")
        @JSONField(name = "modelpath")
        private String modelPath;

        @NestedConfigurationProperty
        private BootDeploySystemSetting settings;
    }


    @Data
    @NoArgsConstructor
    @AllArgsConstructor
    public static class BootDeploySystemSetting {
        @JsonProperty("sysdbscheme")
        @JSONField(name = "sysdbscheme")
        private Map<String, SysDBScheme> sysDBScheme;
        public void addSysDBScheme(String key,SysDBScheme value) {
            if(sysDBScheme==null)
                sysDBScheme=new LinkedHashMap<>();
            sysDBScheme.put(key,value);
        }


        @JsonProperty("sysbdscheme")
        @JSONField(name = "sysbdscheme")
        private Map<String, SysBDScheme> sysBDScheme;
        public void addSysBDScheme(String key,SysBDScheme value) {
            if(sysBDScheme==null)
                sysBDScheme=new LinkedHashMap<>();
            sysBDScheme.put(key,value);
        }

        @JsonProperty("subsysserviceapi")
        @JSONField(name = "subsysserviceapi")
        @NestedConfigurationProperty
        private SubApis subSysServiceApi;

        @JsonProperty("cloudclientutil")
        @JSONField(name = "cloudclientutil")
        @NestedConfigurationProperty
        private SubSysServiceApi cloudClientUtil;

        @JsonProperty("sysutil")
        @JSONField(name = "sysutil")
        @NestedConfigurationProperty
        private SysUtil sysUtil;
        public void addSysUtil(String key,Map value) {
            if(sysUtil==null)
                sysUtil=new SysUtil();
            sysUtil.put(key,value);
        }
    }

    @Data
    @NoArgsConstructor
    @AllArgsConstructor
    public static class SysDBScheme {
        @JsonProperty("datasource")
        @JSONField(name = "datasource")
        private String dataSource;

        @JsonProperty("dbtype")
        @JSONField(name = "dbtype")
        private String dbType;
    }

    public static class SubApis extends LinkedHashMap<String,SubApiModule>{

    }

    public static class SubApiModule extends LinkedHashMap<String,SubSysServiceApi>{

    }

    @Data
    @NoArgsConstructor
    @AllArgsConstructor
    public static class SubSysServiceApi {
        @JsonProperty("serviceurl")
        @JSONField(name = "serviceurl")
        private String serviceUrl ;

        @JsonProperty("globalheadermap")
        @JSONField(name = "globalheadermap")
        private Map<String, Object> globalHeaderMap;

        @JsonProperty("defaultheadermap")
        @JSONField(name = "defaultheadermap")
        private Map<String, Object> defaultHeaderMap;

        @JsonProperty("accesstokenurl")
        @JSONField(name = "accesstokenurl")
        private String accessTokenUrl;

        @JsonProperty("authmode")
        @JSONField(name = "authmode")
        private String authMode;

        @JsonProperty("clientid")
        @JSONField(name = "clientid")
        private String clientId = null;

        @JsonProperty("clientsecret")
        @JSONField(name = "clientsecret")
        private String clientSecret = null;

        @JsonProperty("tokentimeout")
        @JSONField(name = "tokentimeout")
        private Long tokenTimeout;

        @JsonProperty("defaulttokentimeout")
        @JSONField(name = "defaulttokentimeout")
        private Integer defaultTokenTimeout;
    }

    @Data
    @NoArgsConstructor
    @AllArgsConstructor
    public static class SysBDScheme {
        @JsonProperty("serviceurl")
        @JSONField(name = "serviceurl")
        private String serviceUrl;
    }

    public static class SysUtil extends LinkedHashMap<String,Map>{

    }


}
