package net.ibizsys.central.plugin.boot.core.runtime;

import com.alibaba.cloud.nacos.NacosDiscoveryProperties;
import com.baomidou.dynamic.datasource.creator.DataSourceCreator;
import com.baomidou.dynamic.datasource.spring.boot.autoconfigure.DataSourceProperty;
import net.ibizsys.central.cloud.core.ServiceHubBase;
import net.ibizsys.central.cloud.core.service.ISysServiceAPIDocAdapter;
import net.ibizsys.central.cloud.core.util.domain.DataSource;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.boot.autoconfigure.web.ServerProperties;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.context.ConfigurableApplicationContext;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Component;
import org.springframework.web.servlet.mvc.method.annotation.RequestMappingHandlerMapping;

import javax.annotation.PostConstruct;
import java.sql.Connection;

@Component("ServiceHub")
public class BootServiceHub extends ServiceHubBase implements ApplicationContextAware,ApplicationRunner{

	private static final org.apache.commons.logging.Log log = org.apache.commons.logging.LogFactory.getLog(BootServiceHub.class);
	
	@Autowired
    BootServiceHubSetting iServiceHubSetting;
	
	@Autowired
	RequestMappingHandlerMapping requestMappingHandlerMapping;
	
	@Autowired(required=false)
    ISysServiceAPIDocAdapter sysServiceAPIDocAdapter = null;
	
	

	@Autowired 
	NacosDiscoveryProperties nacosDiscoveryProperties;
	
	@Autowired 
	ServerProperties serverProperties;
	

	
	
	@Autowired(required=false)
	javax.sql.DataSource dataSource = null;

	@Autowired
	@Lazy
	private DataSourceCreator druidDataSourceCreator;

	private ConfigurableApplicationContext ctx;
    
    @Override
    public void setApplicationContext(ApplicationContext ctx) throws BeansException {
        this.ctx = (ConfigurableApplicationContext) ctx;
    }
    

    
	@PostConstruct
	public void postConstruct() {
		this.setServiceHubSetting(iServiceHubSetting);
		this.setDefaultDataSource(dataSource);
		this.setRequestMappingHandlerMapping(requestMappingHandlerMapping);
		this.setSysServiceAPIDocAdapter(sysServiceAPIDocAdapter);
		onPostConstruct();
	}


	protected void registerDataSource(DataSource ds) throws Exception{

		DataSourceProperty dataSourceProperty = new DataSourceProperty();
		dataSourceProperty.setDriverClassName(ds.getDriverClassName());
		dataSourceProperty.setUrl(ds.getJdbcUrl());
		dataSourceProperty.setUsername(ds.getUsername());
		dataSourceProperty.setPassword(ds.getPassword());
		javax.sql.DataSource dataSource = druidDataSourceCreator.createDataSource(dataSourceProperty);

        Connection connection = dataSource.getConnection();
        ctx.getBeanFactory().registerSingleton("dynadatasource_"+ds.getDataSourceId(), dataSource);
        try {
        	connection.close();
        }
        catch(Exception ex) {
        	log.error(ex);
        }
        this.setDataSource(ds.getDataSourceId(), dataSource);
	}



	@Override
	public void run(ApplicationArguments args) throws Exception {
		this.install();
	}







	
}
