package net.ibizsys.central.plugin.boot.core.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.util.TypeUtils;
import net.ibizsys.central.dataentity.IDataEntityRuntime;
import net.ibizsys.central.dataentity.IDataEntityRuntimeContext;
import net.ibizsys.central.plugin.boot.core.domain.BaseData;
import net.ibizsys.central.plugin.boot.core.filter.BaseFilter;
import net.ibizsys.central.plugin.boot.core.helper.StringAdvUtils;
import net.ibizsys.central.plugin.boot.core.runtime.IBootService;
import net.ibizsys.central.plugin.boot.core.runtime.IBootSystemRuntime;
import net.ibizsys.central.service.client.IWebClientRep;
import net.ibizsys.central.util.IEntityDTO;
import net.ibizsys.central.util.PageRequest;
import net.ibizsys.model.dataentity.action.IPSDEAction;
import net.ibizsys.model.dataentity.ds.IPSDEDataSet;
import net.ibizsys.runtime.util.DataTypeUtils;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.util.Assert;

import java.io.Serializable;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

public interface IServiceApiService<T extends BaseData, F extends BaseFilter> extends IBaseService<T,F>{


    default void beforeAction(String strActionName,Object arg)  throws Throwable{
        if(arg.getClass().isAssignableFrom(getTClass())) {
            T data=((T)arg);
            data.getFocusNull().forEach(item->{
                data.setObject(item+"dirtyflag",1);
            });
            data.setObject("dirtyflagenable",1);
        }
        try {
            Method beforeMethod = getServiceMethod("before" + StringAdvUtils.pascalcase(strActionName), arg.getClass());
            if (beforeMethod != null)
                beforeMethod.invoke(this, arg);
        }catch (InvocationTargetException ex){
            throw ex.getTargetException();
        }
    }

    default Object onRealAction(String strActionName,Object arg)  throws Throwable{
        return onRealAction(strActionName,arg,getTClass());
    }

    default <K> List<K> onRealActionList(String strActionName,Object arg,Class<K> clazz)  throws Throwable{
        Object obj = this.getDataEntityRuntimeContext().executeActionReal(strActionName, null, new Object[]{arg}, null);
        if(obj instanceof IWebClientRep) {
            String body=((IWebClientRep<String>)obj).getBody();
            Assert.hasText(body,String.format("[{}]接口服务返回为空",strActionName));
            return JSON.parseArray(body,clazz);
        }
        return (List<K>)obj;
    }

    default <K> K onRealAction(String strActionName,Object arg,Class<K> clazz)  throws Throwable{
        Object obj = this.getDataEntityRuntimeContext().executeActionReal(strActionName, null, new Object[]{arg}, null);
        if(obj instanceof IWebClientRep) {
            String body=((IWebClientRep<String>)obj).getBody();
            Assert.hasText(body,String.format("[{}]接口服务返回为空",strActionName));
            K ret = JSON.parseObject(body,clazz);
            if(arg.getClass().isAssignableFrom(getTClass())&&clazz.isAssignableFrom(getTClass()))
                ((T) arg).reload(((T)ret).any(), true);
            return ret;
        }
        return (K)obj;
    }

    default Object onRealFetch(String dsName,F arg) throws Throwable{
        return onRealFetchPage(dsName,arg);
    }

    default List<T> onRealSearch(String dsName,F arg) throws Throwable{
        Object obj = this.getRuntimeService().getDataEntityRuntime().selectDataQuery(dsName, arg);
        if(obj instanceof IWebClientRep) {
            IWebClientRep<String> rep=(IWebClientRep)obj;
            Assert.hasText(rep.getBody(),String.format("[{}]查询服务返回为空",dsName));
            JSON json=(JSON)JSON.parse(rep.getBody());
            if(json instanceof JSONArray) {
                return JSON.parseArray(rep.getBody(),getTClass());
            }
            else if(json instanceof JSONObject){
                if(((JSONObject) json).containsKey("content"))
                    return JSON.parseArray(JSON.toJSONString(((JSONObject) json).getJSONArray("content")),getTClass());
            }
        }
        return (List)obj;
    }

    default Page<T> onRealFetchPage(String dsName,F arg) throws Throwable{
        Object obj = this.getDataEntityRuntimeContext().fetchDataSetReal("DEFAULT", arg.getDataSet(), new Object[]{arg}, null);
        if(obj instanceof IWebClientRep) {
            IWebClientRep<String> rep=(IWebClientRep)obj;
            Assert.hasText(rep.getBody(),String.format("[{}]查询服务返回为空",dsName));
            JSON json=(JSON)JSON.parse(rep.getBody());
            if(json instanceof JSONArray) {
                Long nTotal = DataTypeUtils.getLongValue(rep.getHeader("x-total"), 0L);
                return new PageImpl(JSON.parseArray(rep.getBody(),getTClass()), arg.getPageable(), nTotal);
            }
            else if(json instanceof JSONObject){
                Long nTotal=DataTypeUtils.getLongValue(((JSONObject) json).getLongValue("totalElements"), 0L);
                return new PageImpl(JSON.parseArray(JSON.toJSONString(((JSONObject) json).getJSONArray("content")),getTClass()), arg.getPageable(), nTotal);
            }
        }
        return (Page)obj;
    }

    default T onGetDraft(T dto) throws Throwable {
        return (T)onRealAction("GetDraft",dto);
    }

    default Integer onCheckKey(T dto) throws Throwable {
        return onRealAction("CheckKey",dto,Integer.class);
    }


    default Page<T> onFetchDefault(F dto) throws Throwable {
        return onRealFetchPage("DEFAULT",dto);
    }

    default List<T> selectDefault(F dto) throws Throwable {
        return onRealSearch("DEFAULT", dto);
    }

    default List<T> selectSimple(F dto) throws Throwable {
        return onRealSearch("SIMPLE", dto);
    }

    default List<T> selectView(F dto) throws Throwable {
        return onRealSearch("VIEW", dto);
    }

}
